@@ -4,10 +4,10 @@ use strict;
 use warnings;
 use 5.010;
 
-use Digest::SHA qw(sha1_hex);
+use Digest::SHA;
 use Storable qw(nstore retrieve);
 
-our $VERSION = '1.00';
+our $VERSION = '1.01';
 
 sub new {
 	my ( $obj, %conf ) = @_;
@@ -19,6 +19,7 @@ sub new {
 
 	$ref->{config} = \%conf;
 	$ref->{config}->{read_size} //= ( 2**20 ) * 4;    # 4 MiB
+	$ref->{version} = $VERSION;
 
 	return bless( $ref, $obj );
 }
@@ -28,6 +29,10 @@ sub new_from_file {
 
 	my $ref = retrieve($file);
 
+	if ( not defined $ref->{version} ) {
+		$ref->{version} = '1.00';
+	}
+
 	return bless( $ref, $obj );
 }
 
@@ -35,6 +40,10 @@ sub si_size {
 	my ( $self, $bytes ) = @_;
 	my @post = ( q{ }, qw(k M G T) );
 
+	if ( $bytes == 0 ) {
+		return 'infinite';
+	}
+
 	while ( $bytes >= 1024 ) {
 		$bytes /= 1024;
 		shift @post;
@@ -46,10 +55,15 @@ sub si_size {
 sub hash_file {
 	my ( $self, $file ) = @_;
 	my $data;
+	my $digest = Digest::SHA->new(1);
 
 	# read() fails for empty files
 	if ( ( stat($file) )[7] == 0 ) {
-		return sha1_hex();
+		return $digest->hexdigest;
+	}
+	if ( $self->{config}->{read_size} == 0 ) {
+		$digest->addfile($file);
+		return $digest->hexdigest;
 	}
 
 	#<<< perltidy has problems indenting 'or die' with tabs
@@ -64,7 +78,8 @@ sub hash_file {
 		or die("Can't close ${file}: $!\n");
 
 	#>>>
-	return sha1_hex($data);
+	$digest->add($data);
+	return $digest->hexdigest;
 }
 
 sub hash_in_db {
@@ -96,6 +111,29 @@ sub read_size {
 	return $self->{config}->{read_size};
 }
 
+sub db_info {
+	my ($self) = @_;
+
+	return sprintf(
+		"Database created by hashl v%s\n"
+		  . "Read size: %d bytes (%s)\n"
+		  . "contains: %d file%s and %d ignored hash%s\n",
+		$self->db_version,
+		$self->read_size,
+		$self->si_size( $self->read_size ),
+		scalar( $self->files ),
+		( scalar( $self->files ) == 1 ? q{} : 's' ),
+		scalar( $self->ignored ),
+		( scalar( $self->ignored ) == 1 ? q{} : 'es' ),
+	);
+}
+
+sub db_version {
+	my ($self) = @_;
+
+	return $self->{version};
+}
+
 sub file {
 	my ( $self, $name ) = @_;
 
@@ -199,7 +237,7 @@ App::Hashl - Partially hash files, check if files are equal etc.
 
 =head1 VERSION
 
-This manual documents App::Hashl version 1.00
+This manual documents App::Hashl version 1.01
 
 =head1 DESCRIPTION
 
@@ -220,7 +258,7 @@ Returns a new B<App::Hashl> object. Accepted parameters are:
 =item B<read_size> => I<bytes>
 
 How many bytes of a file to consider for the hash.  Defaults to 4 MiB (4 *
-2**20 bytes).
+2**20 bytes). 0 means read the whole file.
 
 =back
 
@@ -333,7 +371,7 @@ None known.
 
 =head1 AUTHOR
 
-Copyright (C) 2011 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2011-2017 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
